var fs = require('fs');
var path = require('path');
var glob = require('glob');
var crypto = require('crypto')

function fileEtag(file){
  var shasum = crypto.createHash('sha1');
  shasum.update(fs.statSync(file).mtime+":"+file);
  return shasum.digest('hex');
}

function findFile2(directory,langs,file){
  var result = null;
  for(var i=0;i<langs.length;i++){
    var searchExpr = directory+langs[i]+"."+file;
    var files = glob.sync(searchExpr);
    if(!files.length)
      continue;
    result = files[0];
    break;
  }
  return result;
}

function findFile(root,langs,p,success,error){
  var directory = root+"/content/"+path.dirname(p)+"/";
  var file = path.basename("/content/"+p);
  var langs = langs.split(':');
  var f = findFile2(directory,langs,file);
  if(f){
    fs.readFile(f,'utf-8',function(e,s){
      if(e){
        error(404,e.stack);
      }else{
        success(s,f);
      }
    });
  }else{
    error(404,"Not Found");
  }
}

function merge(x,y){
  for (var k in y)
    x[k] = y[k];
}

function findFiles(root,langs,p,ext,success,error){
  var directory = root+"/"+path.normalize("/content/"+(p||""))+"/";
  if(!fs.existsSync(directory)){
    error(404,"Not Found");
    return;
  }
  var pattern = "@("+(langs.replace(/:/g,'|'))+")\\."+(ext?ext.substr(1):'*');
  var search = path.normalize(directory) + pattern;
  var files = glob.sync(search);
  var urls = [];
  var names = [];
  for(var i=0;i<files.length;i++){
    var parts = path.basename(files[i]).match("([^.]*)\\.(.*)");
    var lang = parts[1];
    var name = parts[2];
    if(names.indexOf(name)==-1){
      names.push(name);
      urls.push({
        name: name,
        lang: lang,
        etag: fileEtag(files[i])
      });
    }
  }
  var f = findFile2(directory,langs.split(/:/g),"pageinfo.json");
  var datas = {
    files: urls
  };
  if(f){
    fs.readFile(f, 'utf8', function(err,data){
      if(!err)
        try {
          merge(datas,JSON.parse(data));
        } catch(e){console.error(e.stack);}
      success(datas);
    });
  }else{
    success(datas);
  }
}

module.exports=function(s){
  s.push({
    rule: /^([^\/]+)\/lang\/([a-zA-Z0-9\-_:*]+)\/((.+\/)?(:.+)?)$/,
    handler: function(req,res,url,host,langs,pe,p,ext){
      function sendDatas(datas){
        try {
          res.writeHead(200, {'Content-Type': 'text/plain'});
          res.write(JSON.stringify(datas));
          res.end();
        } catch(e){console.error(e.stack);}
      }
      findFiles(http_root+host,langs,p,ext,sendDatas,function(code,error){
        res.writeHead(code, {'Content-Type': 'text/plain'});
        res.write(error);
        res.end();
      });
    }
  });
  s.push({
    rule: /^([^\/]+)\/lang\/([a-zA-Z0-9\-_:*]+)\/(.*)$/,
    handler: function(req,res,url,host,langs,p){
      var directory = http_root+host+"/content/"+path.dirname(p)+"/";
      var file = path.basename("/content/"+p);
      var langs = langs.split(':');
      var f = findFile2(directory,langs,file);
      if(f){
        var etag = fileEtag(f);
        res.setHeader("ETag",etag);
        if(req.headers['if-none-match'] == etag){
          res.writeHead(304, {"ETag":etag});
          res.end();
        }else{
          var fileStream = fs.createReadStream(f);
          fileStream.pipe(res);
        }
      }else{
        res.writeHead(404, {'Content-Type': 'text/plain'});
        res.write("Not Found");
        res.end();
      }
    }
  });
}

module.exports.findFile = findFile;
module.exports.findFiles = findFiles;
