#!/usr/bin/env node

require("./config.js");

var http = require('http');
//var https = require('https');
var WebSocketServer = require('websocket').server;
var path = require('path');
var fs = require('fs');

process.on('uncaughtException', function(error){
  console.log(error.stack);
});

var services = [];

function loadService(file){
  if(!fs.existsSync(file)){
    console.warn("loadService: file \""+file+"\" not found");
    return;
  }
  try {
    require(file)(services);
  } catch(e){
    console.log(e.stack);
  }
}

loadService('./anyHost.js');
loadService('./games/bomberman/rest.js');
loadService('./pagecontrol.js');

function requestHandler(req, res){
  var host = req.headers.host;
  if(/^www\./.test(host))
    host = host.substr(4);
  var hostValid = /([a-zA-Z0-9äöü\-_]+\.?)+\.([^./]*)/.test(host);
  hostValid = hostValid && fs.existsSync(http_root+host);
  if(!hostValid){
    res.writeHead(500, {'Content-Type': 'text/plain'});
    res.end("Invalid hostname!");
    return;
  }
  try {
    var p = host+path.normalize(req.url);
    handle: {
      for(var i=0;i<services.length;i++){
        if( !services[i].rule )
          continue;
        var m = p.match(services[i].rule);
        if(m){
          services[i].handler.apply(null,[req,res].concat(m));
          break handle;
        }
      }
      res.writeHead(404, {'Content-Type': 'text/plain'});
      res.end("Not Found: "+p);
    };
  } catch(e) {
    try {res.writeHead(500, {'Content-Type': 'text/plain'});}catch(e){}
    res.write(e.stack);
    res.end();
  }
}

var servers = [];
servers.push( http.createServer(requestHandler).listen(3000,'0.0.0.0') );
//servers.push( https.createServer(requestHandler).listen(3001,'0.0.0.0') );

var wsServer = new WebSocketServer({
  httpServer: servers,
  autoAcceptConnections: false
});

function originIsAllowed(host) {
  return fs.existsSync(http_root+host);
}

wsServer.on('request', function(request) {
  var host = request.origin.substr(request.origin.match("^[^:]+://(www.)?")[0].length);
  if (!originIsAllowed(host)) {
    request.reject(400, "bad origin");
    console.log((new Date()) + ' Connection from origin ' + request.origin + ' rejected.');
    return;
  }

  var p = host + path.normalize( request.resource );
  var protocol = null;
  var handler = null;
  var params = null;

  handle: {
    for( var i=0; i < services.length; i++ ){
      var entry = services[i];
      if( !entry.wsrule || !entry.protocols )
        continue;
      params = p.match( entry.wsrule );
      if(!request.requestedProtocols.some(function(proto){
        if(!( proto in entry.protocols ))
          return false;
        protocol = proto;
        handler  = entry.protocols[proto];
        return true;
      })) continue;
      break handle;
    }
    request.reject(404, "Not found");
    return;
  }

  var connection = request.accept( protocol, request.origin );
  console.log( (new Date()) + ' Connection accepted. Protocol: '+ protocol + ', origin: ' + request.origin );
  var instance = handler.apply(null,[connection,request].concat(params));
  for( var event in instance.events ){
    connection.on(event,instance.events[event]);
  }

});
