"use strict";

var Permission = {
  READ : 1<<0,
  WRITE: 1<<1
};

var Remote = (function(){

function RessourcePool( remote ){

  var pool;
  var nextId = 0;
  var ressources = [];

  function create( namespace, permission ){
    var res = new Ressource( pool, namespace, permission );
    res.id = nextId++;
    ressources.push( res );
    return res;
  }

  function get( id ){
    if( x instanceof Object )
      x = x.valueOf();
    if( isNaN(x) || typeof x != "number" )
      return false;
    var result = ressources.findIndex(function(e){return e==x;});
    return result;
  }

  function remove( x ){
    if( x instanceof Object )
      x = x.valueOf();
    if( isNaN(x) || typeof x != "number" )
      return false;
    var result = ressources.findIndex(function(e){return e==x;});
    if( result<0 )
      return false;
    ressources.splice(result,1);
    return true;
  }

  function requestRessourceLock( x, success, error ){
    if( x instanceof Object )
      x = x.valueOf();
    if( isNaN(x) || typeof x != "number" )
      return false;
    remote.requestRessourceLock( x, success, error );
  }

  return pool = {
    create: create,
    get: get,
    remove: remove,
    requestRessourceLock: requestRessourceLock
  };

}

function Ressource( pool, namespace, permission ){
  Datas.prototype = null;
  var datas = Ressource.prototype = new Datas();
  function Datas(){}
  function Ressource(){

    var ressource = this;

    var id = NaN;
    var referenceCount = 1;
    var lockRequests = [];

    function define( key, value ){
      Object.defineProperty(ressource,key,{
        enumerable: false,
        configurable: false,
        writable: false,
        value: value
      });
    }

    function aquire(){
      ++referenceCount;
    }

    function release(){
      if(!referenceCount)
        return;
      if(!--referenceCount)
        pool.remove();
    }

    function get( key ){
      return datas[key];
    }

    function lock( success, error ){
      if( !lockRequests.length )
        pool.requestRessourceLock( ressource, success, error );
      lockRequests.push({ 
        success: success,
        error: error
      });
    }

    define( "valueOf" ,function(){return id;} );
    define( "toString",function(){return "#"+id;} );
    define( "aquire", aquire );
    define( "release", release );
    define( "get", get );
    define( "lock", lock );

    Object.defineProperty(this,"id",{
      enumerable: false,
      configurable: false,
      get: function(){return id;},
      set: function(x){id=x;}
    });

    Object.freeze(this);
    Object.seal(this);
  }
  return new Ressource();
}

function Remote(){

  var ws;
  var cid;
  var callbacks;
  var remote = {
    addCommand: addCommand,
    removeCommand: removeCommand,
    sendCmd: sendCmd,
    setWebSocket: setWebSocket,
    getRessourcePool: getRessourcePool,
    requestRessourceLock: requestRessourceLock
  };
  var rp = RessourcePool(remote);
  var commands = new Map();

  reset();

  function reset(){
    if(callbacks)
      for(var k in callbacks)
        callbacks[k].error({reason:"reset"});
    callbacks = {};
    cid = 0;
    ws = null;
  }

  function setWebSocket( x ){
    ws = x;
    ws.addEventListener("message", function( msg ){
      var res = JSON.parse( msg.data );
      switch( res.action ){
        case "command": {
          execLocal( res, sendResult.bind(null,res.id,true), sendResult.bind(null,res.id,false) );
        } break;
        case "result": {
          execCallback( res, true );
        } break;
        case "error": {
          execCallback( res, false );
        } break;
      }
    });
    ws.addEventListener("close", reset);
  }

  function execCallback( res, b ){
    if( res.callback in callbacks )
      PC_decode( res.params, callbacks[ res.callback ][b?"success":"error"] );
    delete callbacks[ res.callback ];
  }

  function execLocal( obj, success, error ){
    if(!obj) return;
    if( commands.get( obj.cmd ) instanceof Function ){
      PC_decode( obj.params, function(){
        commands.get( obj.cmd ).apply(null,arguments)( success, error );
      });
    }else{
      console.warn("Command \""+obj.cmd+"\" not found");
    }
  }

  function sendCmd( cmd ){
    return function(){
      var params = Array.prototype.slice.apply( arguments );
      return function( success, error ){
        var id = null;
        if( success || error ){
          id = cid++;
          callbacks[id] = {
            success: success||function(){},
            error: error||function(){}
          };
        }
        ws.send(JSON.stringify({
          action: "command",
          id: id,
          cmd: cmd,
          params: PC_encode( params )
        }));
      };
    };
  }

  function requestRessourceLock( x, success, error ){
    sendCmd( "Ressource.lock" )(x)( success, error );
  }

  addCommand("Ressource.lock", function( ressource ){
    return function( error, success ){
      console.log( "Ressource:", ressource, error, success );
    };
  });

  function sendResult( id, b, result ){
    if( id === null )
      return;
    ws.send(JSON.stringify({
      action: b ? "result" : "error",
      id: id,
      params: PC_encode( params )
    }));
  }

  function addCommand( cmd, func ){
    commands.set( cmd, func );
  }

  function removeCommand(){
    commands.delete( cmd );
  }

  function PC_encode( x ){
    return JSON.stringify( x );
  }

  function PC_decode( x, callback ){
    callback.apply( null, JSON.parse( x ) );
  }

  function getRessourcePool(){
    return rp;
  }

  return remote;
}

  return Remote;
})();