"use strict";

var fs = require('fs');

var ff = require('../anyHost.js');
var dpaxml = require('../dpaxml.js');

module.exports = function(x){
  x.noscript = new NoScript();
}

function cloneNode( o, p ) {
  var r = {
    nodeName: o.nodeName,
    attribute: {},
    parentNode: p || null
  };
  if(o.content)
    r.content = o.content;
  for( var k in o.attribute )
    r.attribute[k] = o.attribute[k];
  r.childNodes = o.childNodes.map( (node) => cloneNode(node,r) );
  return r;
}

function removeNode( node ) {
  if(!node.parentNode)
    return;
  var i = node.parentNode.childNodes.indexOf(node);
  if( i != -1 )
    node.parentNode.childNodes.splice(i,1);
  node.parentNode = null;
}

function errFunc(code, msg){
  if( url != "/error/404/" ){
    sendResponseHeader({
      "Status": "302 Moved Temporarily",
      "Content-Type": "text/plain; charset=utf8",
      "Location": "/error/404/?type=noscript"
    });
    write(code+' '+msg);
  }else{
    sendResponseHeader({
      "Content-Type": "text/plain; charset=utf8",
    });
    write("500 Internal Server Error\n");
    write(code+' '+msg);
  }
}

class NoScript {

  constructor(){
    this.views = new Map();
    this.pending = 0;
  }

  readView( file, success, error ){
    var self = this;
    self.views;
    fs.readFile( file, 'utf8', function( e, string ){
      if( e )
        if( error ){
          error(e);
          return;
        }else throw e;
      var result = dpaxml.parseXML()( string );
      success( result );
    });
  }

  pendinc(){
    this.pending++;
  }

  penddec( done ){
    if(!--this.pending)
      done();
  }

  getTemplate( templ, done, success, error ){
    var self = this;
    var res = self.views.get(templ);
    if( res ){
      success(res);
    }else{
      self.readView( root + '/templates/' + templ + '.html', function( tree ){
        self.processTraverseAll( {}, tree, done );
        var res = self.views.get(templ);
        if( res ){
          success(res);
        }else{
          error(404,"Not Found");
        }
      });
    }
  }

  processTraverse( model, node, done ){
    if( !node||node.nodeName == '#text' )
      return;
    var self = this;
    function suTrav( m, templ, parent ){
      if(!m) return;
      self.pendinc();
      var i = parent.childNodes.length;
      parent.childNodes.push(null);
      self.getTemplate(templ,done,function(node){
        node = cloneNode(node);
        self.processTraverse(m,node,done);
        parent.childNodes[i] = node;
        self.penddec(done);
      },function(){
        self.penddec(done);
      });
    }
    if( node.attribute['data-template'] ){
      removeNode(node);
      var templ = node.attribute['data-template'];
      if(node.attribute.class){
        node.attribute.class += ' ' + templ;
      }else{
        node.attribute.class = templ;
      }
      self.views.set( node.attribute['data-template'], node );
      delete node.attribute['data-template'];
      return;
    }
    if( node.attribute['data-map'] ){
      var o = node.attribute['data-map'].split(':');
      var mkey  = o[0];
      var templ = o[1];
      var val = mkey == "this" ? model : model[mkey];
      if( val instanceof Array )
      for( var v of val ){
        suTrav(v,templ,node);
      }else if(val){
        suTrav(val,templ,node);
      }
      delete node.attribute['data-map'];
    }
    if( 'data-content' in node.attribute ){
      var val = model[node.attribute['data-content']];
      if(!val){
        node.childNodes = [];
      }else if( typeof val == 'string' ){
        node.childNodes=[{
          nodeName: '#text',
          childNodes: [],
          parentNode: node,
          content: val
        }];
      }else{
        node.childNodes = node.childNodes.concat(val);
      }
      delete node.attribute['data-content'];
    }
    if( node.attribute['data-bind'] ){
      var os = node.attribute['data-bind'].split('¦');
      for( var ov of os ){
        var o = ov.split(':');
        var nkey = o[0];
        var mkey = o[1];
        if(model[mkey]){
          node.attribute[nkey] = model[mkey];
        }else{
          delete node.attribute[nkey];
        }
      }
      delete node.attribute['data-bind'];
    }
    if( node.attribute.href && !(/^https?:\/\/|^#/.test(node.attribute.href)) && !/(\?|&)type=noscript/.test( node.attribute.href ) ){
      var url = node.attribute.href.match(/^([^?#]+)?(\?[^#]*)?(#.*)?$/);
      if(url)
        node.attribute.href = url[1] + (url[2]?url[2]+'&':'?') + 'type=noscript' + (url[3]||'');
    }
    self.processTraverseAll(model,node.childNodes,done);
  }

  processTraverseAll(model,nodes,done){
    for( var node of nodes )
      this.processTraverse(model,node,done);
  }

  process( file, model, result, done, error ){
    var self = this;
    self.pendinc();
    self.readView( file, function( tree ){
      self.processTraverseAll( model, tree, done );
      result( tree );
      self.penddec( done );
    },error);
  }

  render(){

    var self = this;
    var file = root + "/root/noscript.html";
    var path = url;
    if(path[url.length-1]!='/')
      path += '/';
    if(path=='/')
      path='/home/';

    ff.findFile( root, langs, "heading.json", function( str ){
      var model = JSON.parse( str );
      ff.findFiles(root,langs,path,".html",function(results){

        var res;
        var n = results.files.length+1;

        function result( result ){
          res = result;
        }

        function done(){
          var xml = dpaxml.serializeXML( res );
          sendResponseHeader({
            "Content-Type": "application/xhtml+xml; charset=utf8"
          });
          write('<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">');
          write( xml );
        }

        function dec(){
          if( !--n )
            self.process( file, model, result, done, errFunc );
        }

        model.title = results.title || 'Daniel Abrecht\'s Homepage';
        model.entries = results.files.map(function( res ){
          var id = res.name.substr(0,res.name.length-5);
          var o = {
             id: id,
             anchor: '#'+id,
             lang: res.lang
          };
          ff.findFile(root,langs,path+res.name,function(results){
            results = dpaxml.parseXML()( results );
            if( results && results[0].nodeName == "script" ){
              var script = results.splice(0,1)[0];
              var s = JSON.parse(script.childNodes[0].content);
              o.title = s.title;
            }
            o.content = results;
            dec();
          },function(code,error){
            o.title = code+'';
            o.content = error;
            dec();
          });
          return o;
        });

        dec();

      },errFunc);
    },errFunc);

  }

}
