window.model = window.model || {};
window.pagechangelisteners = window.pagechangelisteners||[];

var languages = [navigator.language || navigator.userLanguage];
if("languages" in navigator){
  if(navigator.languages.indexOf(languages[0])==-1){
    languages = languages.concat(navigator.languages);
  }else{
    languages = navigator.languages;
  }
}

var langStr = languages.join(":");
langStr.replace(/[^A-Za-z0-9\-._]/g,'');

var svgEl = document.createElementNS("http://www.w3.org/2000/svg", "svg");
var PXpCM = 10 / Math.sqrt(
  Math.pow(svgEl.screenPixelToMillimeterX,2) + 
  Math.pow(svgEl.screenPixelToMillimeterY,2)
);
document.documentElement.style.fontSize = PXpCM+"px";

function loadContent(what,callback,progress){
  var xhr = new XMLHttpRequest();
  xhr.open("GET","/rest/lang/"+langStr+":*/"+what.replace(/^\/*/,''),true);
  xhr.onload = function(){
    if(this.status!=200)
      goToPage("/error/"+this.status+"/",true);
    callback(this.responseText);
    window.pagechangelisteners.forEach(function(x){x();});
  };
  if(progress)
    xhr.addEventListener("progress", function(event){
      if(!event.lengthComputable)
        return;
      progress( 100 * event.loaded / event.total );
    }, false);
  xhr.send();
}

var lastPath;
function loadPage(){
  var loc = location.href.match(/(^[^#]*)/)[0];
  if( lastPath == loc )
    return;
  lastPath = loc;
  var baseUrl = location.pathname;
  if(baseUrl=="/")
    baseUrl += "home/";
  baseUrl = baseUrl.substr(1);
  model.title = "Loading...";
  window.pagechangelisteners.forEach(function(x){x();});
  loadContent(baseUrl+":*.html",pageEntries.bind(null,baseUrl,model,"entries"));
}

loadContent("heading.json",jsonContent.bind(null,model));
loadPage();

function jsonContent(obj,json){
  var o = JSON.parse(json);
  for(var i in o)
    obj[i] = o[i];
}

function pageEntries(baseUrl,obj,prop,json){
  var loadingEntry = document.createElement("div");
  var loadingIcon = document.createElement("img");
  var fileNameSpan = document.createElement("span");
  loadingIcon.src = "/images/loading.gif";
  loadingIcon.align = "middle";
  loadingEntry.className = "loading";
  fileNameSpan.setAttribute("data-content","url");
  loadingEntry.setAttribute("align", "middle");
  loadingEntry.appendChild( loadingIcon );
  loadingEntry.appendChild( document.createTextNode(" file: ") );
  loadingEntry.appendChild( fileNameSpan );
  obj[prop] = [];
  var datas = JSON.parse(json);
  var o = datas.files;
  model.title = datas.title || model.title;
  for(var i=0;i<o.length;i++){(function(i){
    var file = o[i];
    var en = obj[prop][i] = {
      content: loadingEntry.cloneNode(true),
      url: location.origin+"/rest/lang/"+langStr+":*/"+file.name,
    };
    loadContent(baseUrl+file.name,function(html){
      var content = document.createElement("div");
      content.innerHTML = html;
      var entry = content.getElementsByClassName('entryInfo')[0];
      entry = entry.textContent||entry.innerText;
      entry = JSON.parse(entry);
      entry.content = content;
      entry.file = o[i].name;
      entry.lang = o[i].lang;
      entry.id = escape(entry.file.substr(0,entry.file.lastIndexOf('.')));
      entry.anchor = "#"+entry.id;
      obj[prop][i] = entry;
    });
  }).call(this,i);}
}

addEventListener("load",function(){
  var lnk = document.createElement("link");
  lnk.rel="stylesheet";
  lnk.type = "text/css";
  lnk.href = "/css/images.css";
  document.head.appendChild(lnk);
});

addEventListener("popstate",function(event) {
  loadPage();
},true);

function handleExternalLinkClick(target){
  console.log("Navigate to external page: "+target);
  return true;
}

function handleInternalLinkClick(target){
  if(target.getAttribute("data-make-request"))
    return true;
  return goToPage(target.href);
}

addEventListener("click",function(event){
  var target = event.target;

  while(!(
       target instanceof HTMLAnchorElement
    || target instanceof HTMLAreaElement
  ) && (
    target = target.parentElement
  ));
  if(!target)
    return true; // not a link

  if( location.origin == target.origin ){
    if(handleInternalLinkClick(target))
      return true;
  }else{
    if(handleExternalLinkClick(target))
      return true;
  }
  event.preventDefault();
  return false;
},true);

var ws=null, unloading=false;

var remote = new Remote();
var ressourcePool = remote.getRessourcePool();

initWebsocket();

function initWebsocket(){
  ws = new WebSocket((
    ({
      "http:" : "ws:",
      "https:": "wss:"
    })[location.protocol]
  )+"//"+location.host+"/ws/","pagecontrol");

  ws.onopen = function(){
    console.log( "Websocket connection opened" );
    remote.setWebSocket( ws );
  };
/*
  ws.onmessage = function( msg ){
    console.log( msg.data );
  };
*/
  ws.onclose = function( msg ){
    if( !unloading )
      initWebsocket();
    console.log( "Websocket connection closed" );
    remote.setWebSocket(null);
  };

  ws.onerror = function( e ){
    console.error( (e.error||e).stack );
  };

}

onbeforeunload = function(){
  unloading = true;
};
